<?
#
# Tasks library
#
# Includes various files from this directory based on what
# it has to do
#
# Bogdan Stancescu <bogdan@lanifex.com>, February 2001
# Revision 1.0

// Returns an array of task id's for the specified project.
// Tasks are imbricated in the array. For instance, if the
// task structure in the project is
/*
  1 -+-> 2 -+-> 3 -+-> PROJECT TOP
     |      |      |
  4 -+      |      |
            |      |
         5 -+      |
            |      |
         6 -+      |
                   |
                7 -+
*/
// Then the resulting array will be
/*
  array(
    array("id"=>3,"children"=>array(
      array("id"=>2,"children"=>array(
        array("id"=>1),
        array("id"=>4)),
      array("id"=>5),
      array("id"=>6))),
    array("id"=>7)))
*/
// Actually, at the same level with "id" there's "name" which
// contains the task name, but omitted it from the schema above for clarity.
// If the last parameter is omitted, only active tasks are returned.
function TaskTree($proj,$task=0,$forceInactive=0) {
  $proj=abs($proj);
  $task=abs($task);
  $tree=array();
  if(!$proj) {
    return(array());
  }
  if (!$forceInactive) {
    $XtraSQL="AND active=1";
  } else {
    $XtraSQL="";
  }
  if ($task) {
    $sql="SELECT id,TaskName FROM task WHERE parent=$task $XtraSQL";
  } else {
    $sql="SELECT id,TaskName FROM task WHERE ProjectID=$proj AND parent=0 $XtraSQL";
  }
  $result=OPT_run_query($sql);
  while(list($taskID,$taskName)=OPT_fetch_row($result)) {
    $branch["id"]=$taskID;
    $branch["name"]=$taskName;
    $branch["children"]=TaskTree($proj,$taskID);
    $tree[]=$branch;
  }
  return($tree);
}

// Used internally for TaskTreeDropdown below - returns the actual
// <option> tags.
function TaskTreeDropdownOptions($branch,$level,$selected,$exclude) {
  reset($branch);
  for ($i=0;$level>$i;$i++) {
    $indent.="&nbsp;&nbsp;";
  }
  while(list(,$leaf)=each($branch)) {
    if ($leaf["id"]==$exclude) {
      continue;
    }
    if ($leaf["id"]==$selected) {
      $sel=" selected";
    } else {
      $sel="";
    }
    $text.="  <option value='".$leaf["id"]."'$sel>$indent".$leaf["name"]."</option>\n";
    if ($leaf["children"]) {
      $text.=TaskTreeDropdownOptions($leaf["children"],$level+1,$selected,$exclude);
    }
  }
  return($text);
}

// Uses TaskTree and TaskTreeDropdownOptions above to build a nice indented
// dropdown of tasks. The parameters are the project id, the variable name
// for the dropdown, the text to use for "no task", the id of the task to be
// selected by default and an optional task id.
// If the last task id is given, it and its children will not be shown in the list.
function TaskTreeDropdown($proj, $varName, $noTask, $selected=0, $exclude=0) {
  $tree=TaskTree($proj);
  $result="<select name='$varName'>\n";
  $result.="  <option value='0'>$noTask</option>\n";
  $result.=TaskTreeDropdownOptions($tree,0,$selected,$exclude);
  $result.="</select>\n";
  return($result);
}

// Returns an array containing "1" for the keys equal to
// children of task $id
function WhoCant($id) {
  $sql="select id from task where parent=$id";
  $result=CRM_run_query($sql);
  while (list($child)=OPT_fetch_row($result)) {
    $array=WhoCant($child);
    while (list($key,$val)=each($array)) {
      $children[$key]=1;
    }
  }
  $children[$id]=1;
  return($children);
}

// Accepts any combination of task/project - when one or both are null, the larger scope is used:
// - if both are present, only task is used;
// - if only task is present, it computes the percent complete for that task;
// - if task is 0 and project is passed, the overall project progress is returned;
// - if neither is present, it computes the overall system progress
// Returns an array containing the following (in this order):
// - the total number of tasks involved (including the one specified, if any);
// - the total number of tasks which affect the current task progress;
// - the current progress of this task;
// - the percent of influence this task has over its parent
// - the percent of influence of this task's progress setting for itself
// - the total time for all the tasks below and this one (in seconds)
// - [BERTELSEN]: the estimated task time total for the specified task tree
function compute_progress($task=0, $project=0) {
  if ($task) {
    $condition="parent=$task";
  } elseif ($project) {
    $condition="projectID=$project and parent=0";
  } else {
    $condition="parent=0";
  }
  $sql="select id, PercentOfParent from task where $condition and active=1";
  $result=CRM_run_query($sql);
  while ($myrow=OPT_fetch_row($result)) {
    list($T_totalcount,$T_affectcount,$T_complete,$T_affect,$T_personal,$T_time,$T_estimated)=compute_progress($myrow[0]);
    $totalcount+=$T_totalcount;
    $affectcount+=$T_affectcount;
    $complete+=$T_affect/100*$T_complete;
    $affect+=$T_affect;
    $time+=$T_time;
    $estimated+=$T_estimated;
    if ($myrow[1]) {
      $affect=$myrow[1];
      $affectcount++;
    }
  }
  $personal=100-$affect;
  if ($task) {
    $sql="select id, PercentComplete, PercentOfParent, time_to_sec(estimated) from task where id=$task";
    $result=CRM_run_query($sql);
    $myrow=OPT_fetch_row($result);
    $sql="select sum(time_to_sec(worked)) from task_progress_history where taskID=$task";
    list($loctime)=OPT_fetch_row(CRM_run_query($sql));
    $time+=$loctime;
    $estimated += $myrow[3];
    $complete+=$personal*$myrow[1]/100;
  }
  $totalcount++;
  return(array($totalcount,$affectcount,$complete,$affect,$personal,$time,$estimated));
}

// This function makes sure none of the parents of the specified task
// have start later than this one or end earlier than it.
// When called, the function calls itself supplying the parent as the
// argument.
// Returns the number of changes made (number of affected parents).
function propagate_task($task) {
  $changes=0;
  $task=abs($task);
  if (!$task) {
    return(0);
  }
  $taskData=getvalue(
    "task",array("StartDatePlanned","EndDatePlanned","StartDate","EndDate","parent"),
    $task,0);
  $parent=$taskData[4];
  if (!$parent) {
    return(0);
  }
  $parentData=getvalue(
    "task",array("StartDatePlanned","EndDatePlanned","StartDate","EndDate"),
    $parent,0);
  if ($parentData[0] && $taskData[0] && ($parentData[0]>$taskData[0])) {
    $sql="UPDATE task SET StartDatePlanned=$taskData[0], date=date where id=$parent";
    OPT_run_query($sql);
    $changes++;
  }
  if ($parentData[1] && $taskData[1] && ($taskData[1]>$parentData[1])) {
    $sql="UPDATE task SET EndDatePlanned=$taskData[1], date=date where id=$parent";
    OPT_run_query($sql);
    $changes++;
  }
  if ($parentData[2] && $taskData[2] && ($parentData[2]>$taskData[2])) {
    $sql="UPDATE task SET StartDate=$taskData[2], date=date where id=$parent";
    OPT_run_query($sql);
    $changes++;
  }
  if ($parentData[3] && $taskData[3] && ($taskData[3]>$parentData[3])) {
    $sql="UPDATE task SET EndDate=$taskData[3], date=date where id=$parent";
    OPT_run_query($sql);
    $changes++;
  }
  return($changes+propagate_task($parent));
}

?>
